USER_AGENTS = ['Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:105.0) Gecko/20100101 Firefox/105.0',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/106.0.0.0 Safari/537.36',
               'Mozilla/5.0 (X11; Linux x86_64; rv:105.0) Gecko/20100101 Firefox/105.0',
               'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36',
               'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/106.0.0.0 Safari/537.36',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:104.0) Gecko/20100101 Firefox/104.0',
               'Mozilla/5.0 (Windows NT 10.0; rv:105.0) Gecko/20100101 Firefox/105.0',
               'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:105.0) Gecko/20100101 Firefox/105.0',
               'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/16.0 Safari/605.1.15',
               'Mozilla/5.0 (X11; Linux x86_64; rv:104.0) Gecko/20100101 Firefox/104.0',
               'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36',
               'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/106.0.0.0 Safari/537.36',
               'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:105.0) Gecko/20100101 Firefox/105.0',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:106.0) Gecko/20100101 Firefox/106.0',
               'Mozilla/5.0 (X11; Linux x86_64; rv:102.0) Gecko/20100101 Firefox/102.0',
               'Mozilla/5.0 (X11; Linux x86_64; rv:103.0) Gecko/20100101 Firefox/103.0',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/106.0.0.0 Safari/537.36 Edg/106.0.1370.42',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36 Edg/105.0.1343.42',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36 Edg/105.0.1343.53',
               'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:104.0) Gecko/20100101 Firefox/104.0',
               'Mozilla/5.0 (X11; Linux x86_64; rv:91.0) Gecko/20100101 Firefox/91.0',
               'Mozilla/5.0 (Windows NT 10.0; rv:104.0) Gecko/20100101 Firefox/104.0',
               'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/15.6.1 Safari/605.1.15',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/106.0.0.0 Safari/537.36 Edg/106.0.1370.37',
               'Mozilla/5.0 (Windows NT 10.0; rv:91.0) Gecko/20100101 Firefox/91.0',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36 Edg/105.0.1343.50',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:102.0) Gecko/20100101 Firefox/102.0',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.0.0 Safari/537.36',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.5112.102 Safari/537.36 OPR/90.0.4480.117',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/106.0.0.0 Safari/537.36 Edg/106.0.1370.47',
               'Mozilla/5.0 (X11; Ubuntu; Linux x86_64; rv:104.0) Gecko/20100101 Firefox/104.0',
               'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.0.0 Safari/537.36',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/106.0.0.0 Safari/537.36 Edg/106.0.1370.34',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:91.0) Gecko/20100101 Firefox/91.0',
               'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/16.1 Safari/605.1.15',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.5112.102 Safari/537.36 OPR/90.0.4480.84',
               'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36',
               'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36',
               'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/15.4 Safari/605.1.15',
               'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/15.5 Safari/605.1.15',
               'Mozilla/5.0 (Windows NT 6.1; Win64; x64; rv:105.0) Gecko/20100101 Firefox/105.0',
               'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36',
               'Mozilla/5.0 (Macintosh; Intel Mac OS X 10.15; rv:106.0) Gecko/20100101 Firefox/106.0',
               'Mozilla/5.0 (X11; Linux x86_64; rv:106.0) Gecko/20100101 Firefox/106.0',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:103.0) Gecko/20100101 Firefox/103.0',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:107.0) Gecko/20100101 Firefox/107.0',
               'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/106.0.0.0 Safari/537.36',
               'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/79.0.3945.88 Safari/537.36',
               'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.0.0 Safari/537.36',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.5112.126 Safari/537.36',
               'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/605.1.15 (KHTML, like Gecko) Version/15.6 Safari/605.1.15',
               'Mozilla/5.0 (Windows NT 10.0; rv:103.0) Gecko/20100101 Firefox/103.0',
               'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/105.0.0.0 Safari/537.36 OPR/91.0.4516.65',
               'Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/106.0.0.0 Safari/537.36',
               'Mozilla/5.0 (X11; Fedora; Linux x86_64; rv:100.0) Gecko/20100101 Firefox/100.0'
               ]



from amazondata import user_agents
from selectorlib import Extractor
import requests
import random


class SearchResultExtractor:
    def __init__(self):
        self._amazon_product_extractor = Extractor.from_yaml_file(
            'amazondata/amazon_search_result.yml')

    def __scrape(self, url):
        headers = {
            'dnt': '1',
            'upgrade-insecure-requests': '1',
            'user-agent': random.choice(user_agents.USER_AGENTS),
            'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
            'sec-fetch-site': 'same-origin',
            'sec-fetch-mode': 'navigate',
            'sec-fetch-user': '?1',
            'sec-fetch-dest': 'document',
            'referer': 'www.amazon.in',
            'accept-language': 'en-GB,en-US;q=0.9,en;q=0.8',
        }

        r = requests.get(url, headers=headers)

        if r.status_code > 500:
            raise Exception(
                "Page %s was blocked by Amazon. Please try using better proxies." % url)
        else:
            return self._amazon_product_extractor.extract(r.text)

    def __process_rating(self, data):
        if 'rating' in data:
            rating = data['rating']

            if rating:
                rating = rating.replace('out of 5 stars', '').strip()

            return rating
        return None

    def __process_number_of_ratings(self, data):
        if 'number_of_ratings' in data:
            number_of_ratings = data['number_of_ratings']

            if number_of_ratings:
                number_of_ratings = int(
                    number_of_ratings.replace(',', '').strip())

            return number_of_ratings
        return None

    def __process_is_sponsored(self, data):
        if 'is_sponsored' in data:
            is_sponsored = data['is_sponsored']
            if is_sponsored == 'Sponsored':
                return True

        return False

    def __process_url(self, data):
        if 'url' in data:
            url = data['url']

            if url:
                url = 'https://www.amazon.in'+url

            return url
        return None

    def __process_data(self, data):
        for product in data['products']:
            product['rating'] = self.__process_rating(product)
            product['number_of_ratings'] = self.__process_number_of_ratings(
                product)
            product['is_sponsored'] = self.__process_is_sponsored(product)
            product['url'] = self.__process_url(product)

        return data

    def search(self, query, page=1):
        url = 'https://www.amazon.in/s?k='+query+'&page='+str(page)
        data = self.__scrape(url)
        processed_data = self.__process_data(data)
        return processed_data



from setuptools import setup, find_packages

with open('README.md') as readme_file:
    README = readme_file.read()

setup_args = dict(
    name='amazondata',
    version='0.0.2',
    description='A python package to get amazon product and search data in json form. The package does not require any API keys as it works by scraping the amazon page.',
    long_description_content_type="text/markdown",
    long_description=README,
    license='MIT',
    packages=find_packages(),
    author='Anup Kumar Panwar',
    author_email='1anuppanwar@gmail.com',
    keywords=['Amazon', 'Scraper'],
    url='https://github.com/AnupKumarPanwar/amazondata',
    download_url='https://pypi.org/project/amazondata/'
)

install_requires = [
    'selectorlib',
    'requests'
]

if __name__ == '__main__':
    setup(**setup_args, install_requires=install_requires)



from amazondata import user_agents
from selectorlib import Extractor
import requests
import random


class ProductDetailsExtractor:
    def __init__(self):
        self._amazon_product_extractor = Extractor.from_yaml_file(
            'amazondata/amazon_product.yml')

    def __scrape(self, url):
        headers = {
            'dnt': '1',
            'upgrade-insecure-requests': '1',
            'user-agent': random.choice(user_agents.USER_AGENTS),
            'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9',
            'sec-fetch-site': 'same-origin',
            'sec-fetch-mode': 'navigate',
            'sec-fetch-user': '?1',
            'sec-fetch-dest': 'document',
            'referer': 'www.amazon.in',
            'accept-language': 'en-GB,en-US;q=0.9,en;q=0.8',
        }

        r = requests.get(url, headers=headers)

        if r.status_code > 500:
            raise Exception(
                "Page %s was blocked by Amazon. Please try using better proxies." % url)
        else:
            return self._amazon_product_extractor.extract(r.text)

    def __process_categories(self, data):
        if data['categories']:
            return data['categories'].split(' › ')
        return None

    def __process_mrp(self, data):
        mrp1 = data['mrp1']
        mrp2 = data['mrp2']
        mrp = next((mrp for mrp in [
            mrp1, mrp2] if mrp), None)
        return mrp

    def __process_selling_price(self, data):
        selling_price1 = data['selling_price1']
        selling_price2 = data['selling_price2']
        selling_price = next((selling_price for selling_price in [
            selling_price1, selling_price2] if selling_price), None)
        return selling_price

    def __process_ranks(self, data):
        ranks1 = data['ranks1']
        ranks2 = data['ranks2']
        ranks = next((ranks for ranks in [
            ranks1, ranks2] if ranks), None)

        processed_ranks = []

        if ranks and '#' in ranks:
            while '#' in ranks:
                rank_start = ranks.index('#')
                rank_end = ranks.index(' ', rank_start)

                category_start = rank_end + 3
                category_end = len(ranks)
                if '(' in ranks[category_start:] and '#' in ranks[category_start:]:
                    category_end = min(ranks.index(
                        '(', category_start), ranks.index('#', category_start))
                elif '#' in ranks[category_start:]:
                    category_end = ranks.index('#', category_start)

                processed_ranks.append({
                    'rank': int(ranks[rank_start + 1:rank_end].replace(',', '').strip()),
                    'category': ranks[category_start:category_end].strip()
                })

                ranks = ranks[category_end:]

        return processed_ranks or None

    def __process_rating(self, data):
        rating1 = data['rating1']
        rating2 = data['rating2']
        rating = next((rating for rating in [
            rating1, rating2] if rating), None)

        if rating:
            rating = rating.replace('out of 5 stars', '').strip()

        return rating

    def __process_attributes(self, data):
        processed_properties = []

        for i in range(1, 3):
            attribute_keys = data['attribute_keys'+str(i)]

            if attribute_keys:
                values = data['attribute_values'+str(i)]

                if len(values) > len(attribute_keys):
                    values = values[len(values)-len(attribute_keys):]

                for j in range(len(attribute_keys)):
                    property = {
                        'name': str(attribute_keys[j]),
                        'value': str(values[j])
                    }
                    processed_properties.append(property)

        return processed_properties

    def __process_product_details(self, data):
        processed_properties = []

        for i in range(1, 3):
            product_details_keys = data['product_details_keys'+str(i)]

            if product_details_keys:
                values = data['product_details_values'+str(i)]
                if len(values) > len(product_details_keys):
                    values = values[len(values)-len(product_details_keys):]

                for j in range(len(product_details_keys)):
                    property = {
                        'name': str(product_details_keys[j]).replace('\n', '').replace(':', '').replace('\u200e', '').replace('\u200f', '').strip(),
                        'value': str(values[j])
                    }
                    processed_properties.append(property)

        return processed_properties

    def __process_store(self, data):
        store = data['store'].replace(
            'Visit the', '').replace('Store', '').replace('Brand:', '').strip()
        return store

    def __process_number_of_ratings(self, data):
        number_of_ratings = data['number_of_ratings'].replace(
            'ratings', '').strip()
        return number_of_ratings

    def __clean_data(self, data):
        data.pop('selling_price1', None)
        data.pop('selling_price2', None)
        data.pop('mrp1', None)
        data.pop('mrp2', None)
        data.pop('attribute_keys1', None)
        data.pop('attribute_keys2', None)
        data.pop('attribute_values1', None)
        data.pop('attribute_values2', None)
        data.pop('rating1', None)
        data.pop('rating2', None)
        data.pop('ranks1', None)
        data.pop('ranks2', None)
        data.pop('product_details_keys1', None)
        data.pop('product_details_keys2', None)
        data.pop('product_details_values1', None)
        data.pop('product_details_values2', None)

        return data

    def __process_data(self, data):
        data['categories'] = self.__process_categories(data)
        data['mrp'] = self.__process_mrp(data)
        data['selling_price'] = self.__process_selling_price(data)
        data['ranks'] = self.__process_ranks(data)
        data['rating'] = self.__process_rating(data)
        data['key_features'] = self.__process_attributes(data)
        data['product_details'] = self.__process_product_details(data)
        data['store'] = self.__process_store(data)
        data['number_of_ratings'] = self.__process_number_of_ratings(data)

        data = self.__clean_data(data)

        return data

    def get_product_from_url(self, url):
        data = self.__scrape(url)
        processed_data = self.__process_data(data)
        return processed_data

    def get_product_from_asin_code(self, asin_code):
        url = 'https://www.amazon.in/dp/'+asin_code.upper()
        data = self.__scrape(url)
        processed_data = self.__process_data(data)
        return processed_data
